import hashlib
import json

from requests import Session
import getpass


class EschoolBase:
    def __init__(self, login=None, password=None, handled_homeworks=None, handled_msgs=None, handled_marks=None,
                 filename=None, period='204184', user_id=None):
        self.username = login
        self.session = Session()
        self.password = password
        self.handled_marks = handled_marks or []
        self.handled_msgs = handled_msgs or []
        self.handled_homeworks = handled_homeworks or []
        self.period = period
        self.homework_handler = None
        self.mark_handler = None
        self.message_handler = None
        self.user_id = user_id
        self.filename = filename

    def auth(self):
        self.session.post('https://app.eschool.center/ec-server/login',
                          data={'username': self.username,
                                'password': self.password})
        self.user_id = self.session.get('https://app.eschool.center/ec-server/student/diary').json()['user'][0]['id']

    @classmethod
    def login(cls, login, password=None, period='204184', filename=None):
        """
        Login to the account
        """
        self = cls(period=period, filename=filename)
        password = password or getpass.getpass('Eschool password: ')
        self.username = login
        self.password = hashlib.sha256(password.encode()).hexdigest()
        self.auth()
        return self

    def save(self, filename='eschool_account'):
        """
        Save account to file
        :param filename: filename
        """
        self.filename = filename
        with open(filename, 'w') as f:
            f.write(json.dumps(((self.username, self.password), self.handled_homeworks, self.handled_msgs,
                                self.handled_marks, self.user_id)))

    @classmethod
    def from_file(cls, filename):
        """
        Restore from file
        :param filename: filename
        :return: session
        """
        with open(filename) as f:
            (login, password), homeworks, msgs, marks, user_id = json.loads(f.read())
        self = cls(login, password, homeworks, msgs, marks, filename=filename, user_id=user_id)
        return self

    def get(self, method, prefix='student', **kwargs):
        resp = self.session.get(
            f'https://app.eschool.center/ec-server/{prefix}/{method}?userId={self.user_id}' +
            (f'&eiId={self.period}' if self.period and prefix == 'student' else '') + ('&' if kwargs else '') + '&'.join(
                [key + '=' + str(kwargs[key]) for key in kwargs.keys() if key != 'prefix']))
        if resp.status_code == 401:
            self.auth()
            return self.get(method, prefix, **kwargs)
        resp.raise_for_status()
        return resp.json()

    def put(self, method, data, url_data='', prefix='chat'):
        resp = self.session.put(
            f'https://app.eschool.center/ec-server/{prefix}/{method}{"?" + url_data if url_data else ""}',
            json.dumps(data))
        if resp.status_code == 401:
            self.auth()
            return self.put(method, data, url_data, prefix)
        resp.raise_for_status()
        return resp.json()

    def on_homework(self, func):
        """
        Decorator for handling event (adding homework)
        """
        self.homework_handler = func

    def on_mark(self, func):
        """
        Decorator for handling event (adding mark)
        """
        self.mark_handler = func

    def on_message(self, func):
        """
        Decorator for handling event (adding message)
        """
        self.message_handler = func

    def download_file(self, file_id):
        """
        Download file with file_id
        :param file_id: file id
        :return: raw file content (bytes)
        """
        result = self.session.get(f'https://app.eschool.center/ec-server/files/{file_id}')
        return result.content



import time
import traceback
from threading import Thread

from .eschool_base import EschoolBase


def exception_handler(func):
    def new_func(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print('exception  in handler:', e)
    return new_func


class EschoolClient(EschoolBase):
    """
    Eschool client
    """
    def diary_units(self) -> dict:
        """
        Get diary units (subjects)
        """
        return self.get('getDiaryUnits')

    def marks(self) -> list:
        """
        Get all marks for current period
        :return: list of marks in format:
        [mark value (1,2,3,4,5), mark weight, mark date, lesson (where mark is) id, work name (homework, exam etc),
        lesson name (Algebra etc)]] for  each mark
        """
        result = self.get('getDiaryUnits')['result']
        units = {unit['unitId']: unit['unitName'] for unit in result if unit.get('unitName')}
        result = self.get('getDiaryPeriod')['result']
        return [[lesson['markVal'], lesson.get('mktWt', 1), lesson['startDt'], lesson['lessonId'], lesson['lptName'],
                 units.get(lesson['unitId'])]
                for lesson in result if lesson.get('markVal')]

    def diary(self, d1=None, d2=None):
        """
        Get diary (all lessons)
        :param d1: start date (format - time seconds since the Epoch)
        :param d2:
        :return: list of lessons (each lesson is a dictionary)
        """
        return self.get('diary', d1=(d1 or int(time.time() - 48 * 3600)) * 1000,
                        d2=(d2 or int((d1 or int(time.time() - 48 * 3600)) + 14 * 24 * 3600)) * 1000)['lesson']

    def homeworks(self, d1=None, d2=None):
        """
        Get all homeworks from d1 to d2
        :return: list of homeworks:
        [homework id, lesson name, lesson date, homework text, [file id, filename] for each attachment]
            for each homework
        """
        result = []
        for lesson in self.diary(d1, d2):
            part = list(filter(lambda part: len(part['variant']), lesson.get('part')))
            if not part:
                continue
            part = part[0]
            if not (part['variant'][0].get('text') or part['variant'][0].get('file')):
                continue
            result.append([part['variant'][0]['id'], lesson['unit']['name'], lesson['date'],
                           part['variant'][0].get('text'),
                           [[file['id'], file['fileName']] for file in part['variant'][0]['file']]])
        return result

    def chats(self, chat_count=250):
        """
        Get first chat_count chats
        :return: list of chats (each chat is a dictionary)
        """
        return self.get('threads', prefix='chat', newOnly='false', row=1, rowsCount=chat_count)

    def messages(self, thread_id):
        """
        Get messages from thread with thread_id
        :return: list of messages (dictionaries)
        """
        return self.get('messages', prefix='chat', getNew='false', isSearch='false', rowStart=1, rowsCount=3,
                        threadId=thread_id)

    def chat_members(self, thread_id):
        """
        Get list of chat members
        :param thread_id: id of the chat
        :return: list of members
        """
        return self.get('mem_and_cnt', prefix='chat', threadId=thread_id)['members']

    def get_groups(self):
        """
        Get all available groups
        :return: list of groups
        """
        return self.get('olist', prefix='usr')

    def handling(self):
        """
        Start handling events
        """
        self.handled_homeworks = self.handled_homeworks or [homework[0] for homework in self.homeworks()]
        self.handled_msgs = self.handled_msgs or \
                            [msg['msgId'] for chat in self.chats() for msg in self.messages(chat['threadId'])]
        self.handled_marks = self.handled_marks or [mark[3] for mark in self.marks()]

        def homeworks_and_marks():
            while True:
                try:
                    if self.homework_handler:
                        homeworks = self.homeworks()
                        for homework in homeworks:
                            if homework[0] not in self.handled_homeworks:
                                self.homework_handler(homework)
                                self.handled_homeworks.append(homework[0])
                    if self.mark_handler:
                        marks = self.marks()
                        for mark in marks:
                            if mark[3] not in self.handled_marks:
                                self.mark_handler(mark)
                                self.handled_marks.append(mark[3])
                    time.sleep(60 * 3)
                except Exception as e:
                    print(f'time: {time.time()}, exception: {"".join(traceback.format_tb(e.__traceback__))}')
                if self.filename:
                    self.save()

        def messages_loop():
            while True:
                try:
                    msgs = list(filter(lambda msg: msg['msgId'] not in self.handled_msgs,
                                       [msg for chat in self.chats() for msg in
                                        self.messages([chat['threadId'], time.sleep(30)][0])]))
                    for msg in reversed(msgs):
                        self.handled_msgs.append(msg['msgId'])
                        self.message_handler(msg)
                    time.sleep(60 * 3)
                except Exception as e:
                    print('exception:', e)
                if self.filename:
                    self.save()

        Thread(target=homeworks_and_marks).start()
        if self.message_handler:
            Thread(target=messages_loop).start()
        while True:
            pass

    def send_message(self, thread_id, text):
        """
        Send message with text to chat thread_id
        """
        return self.get('sendNew', prefix='chat', threadId=thread_id, msgText=text)

    def create_chat(self, subject: str, members=()):
        thread_id = self.put('saveThread', {'subject': subject, 'isGroup': 2})
        if members:
            self.put('setMembers',
                     [{'memberObjId': member, 'memberCode': 'PRS', 'memberId': None} for member in members],
                     url_data=f'threadId={thread_id}')
        return thread_id



from setuptools import setup

setup(
    name='eschool',
    version='1.2.14',
    packages=['eschool'],
    package_dir={'eschool': 'eschool'},
    url='https://github.com/ennucore/eschool_py',
    license='MIT',
    author='ennucore',
    author_email='ennucore@gmail.com',
    install_requires=['requests'],
    description='Python library to work with Eschool electronic diary'
)
